<?php

require_once "OaiImport.class.php";

/**
 * Import journals from cyberleninka
 * WARNINIG: cyberleninka can block you server.
 * Usage for import journal:
 *   $o = new Cyberleninka();
 *   $o->rootCatalogNode = node_load('17847');
 *   $journal_url = 'https://cyberleninka.ru/journal/n/vestnik-permskogo-universiteta-filosofiya-psihologiya-sotsiologiya';
 *   $o->importAllFromUrl($journal_url);
 * Usage for import PDF:
 *   $o = new Cyberleninka();
 *   $o->rootCatalogNode = node_load('17847');
 *   $ident = 'https://cyberleninka.ru/article/n/o-sootnoshenii-funktsiy-i-struktur-v-pravovoy-sisteme';
 *   $node = $o->importRecord($ident);
 */
class Cyberleninka extends OaiImport {
  public $endPointUrl;
  public $client;
  public $endPoint;
  function __construct($endPointUrl = "https://cyberleninka.ru/oai") {
    parent::__construct($endPointUrl);
  }
  
  public function importRecord($identifier) {
    
    $node = ElisHelper::findNodeByIdentifier($identifier);
    if ($node != false) return $node; // node is exists
    
    // in cyberleninka identifier it is url of page like 'https://cyberleninka.ru/article/n/o-sootnoshenii-funktsiy-i-struktur-v-pravovoy-sisteme'
    $remoteUrl = $identifier; // it is 
    
    $node = $this->getNodeForRecord($identifier);
    
    ElisImportHelper::attachAllowUnprotectedDownloadToNode($node, '1');
    ElisImportHelper::attachOpenAccessToNode($node, '1');
    ElisImportHelper::attachImportServiceToNode($node, 'cyberleninka');
    ElisImportHelper::attachTextToNodeField($node, 'field_import_source', $remoteUrl);
    ElisImportHelper::attachIdentifierToNode($node, $identifier); // identifier from record
    
    $resourcePage = @file_get_contents($remoteUrl); 
    $dom = new DomDocument();
    $dom->loadHTML($resourcePage);
    $xpath = new DOMXPath($dom);
    
    $fileUrl = $this->findMetaContentFromXpath('citation_pdf_url', $xpath);
    
    if($fileUrl) {

      // set or create serial
      $isSerialExistsInDatabase = false; // true if serial in database is finded and exists by issn
      $issn = $this->findMetaContentFromXpath('citation_issn', $xpath);
      $journal_title = $this->findMetaContentFromXpath('citation_journal_title', $xpath);
      $serial_term = false; // term of type serial for this item
      if ($issn && $journal_title) {
        $serial_term = ElisImportHelper::findOrCreateSerialByIdentifierOrTitle($issn, $journal_title);
        if ($serial_term) ElisImportHelper::attachSerialToNode($node, array($serial_term));
      } else {
        if ($issn) {
          $serial_term = ElisSerial::findSerialByIdentifier($issn); // find serial by issn
          if ($serial_term) ElisImportHelper::attachSerialToNode($node, array($serial_term)); // find and attach serial by title
        } else if ($journal_title) {
          if ($serial_term) ElisImportHelper::attachSerialToNode($node, array($journal_title)); // find and attach serial by title
        }
      }
      if ($serial_term == false) die("can not create serial for record of $remoteUrl. May be serial not have journal_title or issn");

      $issue = $this->findMetaContentFromXpath('citation_issue', $xpath);
      if ($issue) ElisImportHelper::attachTextToNodeField($node, "field_issue", $issue);

      $volume = $this->findMetaContentFromXpath('citation_volume', $xpath);
      if ($volume) ElisImportHelper::attachTextToNodeField($node, "field_volume", $volume);

      $publisher = $this->findMetaContentFromXpath('citation_publisher', $xpath);
      if ($publisher)
        ElisImportHelper::attachPublisherToNode($node, array($publisher));      

      $publication_date = $this->findMetaContentFromXpath('citation_publication_date', $xpath);
      if (preg_match("/^\d{4}$/", $publication_date))
        ElisImportHelper::attachTextToNodeField($node, 'field_publish_date', $publication_date);

      $node_chain = $this->createCatalogTree($this->rootCatalogNode, $volume, $issue, $publication_date, $journal_title);
      if (count($node_chain) > 0) {
        $article_catalog = $node_chain[$count($node_chain)-1];
        ElisImportHelper::attachParentCatalogToNode($node, $article_catalog);
      } else {
        die('catalog root not created');
      }
      
      $node = import_from_file_pdf_ebooks($fileUrl, array('node'=>$node));
    }
    return $node;
  }
  
  protected function createCatalogTree($rootNode=false, $volume = false, $issue=false, $data=false, $journal_title = false) {
    $catalogTree = array();

    if ($rootNode == false) $rootNode = $this->rootCatalogNode;
    
    if (false == $rootNode && $journal_title) { // create new node for journal if 
      $rootNode = SubjNodeBuilder::createSubjNode();
      $rootNode->title = $journal_title;
      ElisImportHelper::attachOpenAccessToNode($rootNode, '1');
      node_save($rootNode);
    }
    
    if ($volume) {
      $volume_catalog_name = $rootNode->title . '. ' . $volume_catalog_name;
      $catalogTree[] = $volume_catalog_name;
    }
    if ($issue) {
      $issue_catalog_name = $rootNode->title . '. ' . $issue;
      if ($volume) {
        $issue_catalog_name = $rootNode->title . '. ' . $issue . ' / ' . $volume; // template if issue and volume is exists
      } else { // use year from data if volume is not set
        $year = ElisSerial::extractYearFromString($data);
        if ($year > '1100' && $year < '2050')
          $issue_catalog_name = $rootNode->title . '. ' . $issue . ' / ' . $year;
        
      }
      $catalogTree[] = $issue_catalog_name;
    }
    if (count($catalogTree)) {
      ElisImportHelper::insertTextChainAsCatalogTree($catalogTree, $rootNode);
    }
  }


  /**
   * Import all journal articles from url of same journal
   * @param string $url of journal like https://cyberleninka.ru/journal/n/vestnik-permskogo-universiteta-yuridicheskie-nauki
   */
  public function importAllFromUrl($url) {
    $page = file_get_contents($url);
    
    $dom = new DomDocument();
    $dom->loadHTML($url);
    $xpath = new DOMXPath($dom);
    $canonicalUrl = $this->findCanonicalUrlFromXpath($xpath);
    // if url of journal, is have in script same json object with {"id":2134234,"name":"12 (123)","year":2014},...
    if (preg_match_all('/{"id":(\d+),"name":"(.*)","year":(\d{4})}/U', $page, $matches)) {
      $journals = [];
      foreach ($matches[0] as $i=>$match) {
        $journal = new stdClass();
        $journal->id = $matches[1][$i];
        $journal->issue = $matches[2][$i];
        $journal->year = $matches[3][$i];
        $journals[] = $journal;
      } 
      $articles = [];
      foreach ($journals as $issue) {
        $issueArticles = $this->findArticlesByIssueId($issue->id);
        if ($issueArticles) {
          $articles = array_merge($articles, $issueArticles);
        }
      }
      foreach ($articles as $article) {
        $node = $this->importRecord($article->identifier);
        var_dump($node);exit;
      }
    }
  }
  
  private function findArticlesByIssueId($issueId) {
    $url = 'https://cyberleninka.ru/api/issue/' . $issueId . '/articles';
    $file = file_get_contents($url);
    $json = json_decode($file);;
    $articles = [];
    if ($json) {
      foreach ($json as $doc) {
        $res = new stdClass();
        $res->link = $doc->link;
        $res->identifier = 'https://cyberleninka.ru' . $doc->link;
        $res->name = $doc->name;
        $res->authorsString = $doc->authors;
        $res->year = $doc->year;
        $articles[] = $res;
      }
    }
    return $articles;
  }
  
  protected function findLicenseFromXpath($xpath) {
    $out = false;
    $elements = $xpath->query("//div[@class='top-cc label-cc']");
    if (!is_null($elements)) {
      foreach ($elements as $element) {
        $content = $element->textContent;
        if ($content != false && $content != '') {
          $out = $content;
          if (preg_match('/CC\sBY(.*)/', $content)) { // for cyberleninka
            
          }
        }
      }
    }
    return $out;
  }
}
